//=============================================================================
// BattleSystem.cpp
//=============================================================================
#include "Stdinc.h"
#include "BattleSystem.h"
#include "DisplayManager.h"
#include "Manipulator.h"
#include "CommandManager.h"
#include "NumberAnimation.h"

#include "Character.h"
#include "Enemy.h"
#include "Field.h"
#include "TableAccess.h"
//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CBattleSystem::CBattleSystem(void)
	:m_dwSystemFrameCount( 0 ),m_nMode( MODE_INIT )
{
	// p[eB
	RtlZeroMemory( m_pPlayerChar, sizeof(m_pPlayerChar) );

	// Gp[eB
	RtlZeroMemory( m_pEnemy, sizeof(m_pEnemy) );

	// \}l[W
	m_pDisplayMan = new CDisplayManager();

	// Ǘ
	m_pManipulator = new CManipulator();

	// R}hǗ
	m_pCommandMan = new CCommandManager();

	// tB[h
	m_pField = new CField();
}

//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CBattleSystem::~CBattleSystem(void)
{
	for ( int iCnt = 0; iCnt < BATTLE_CHAR_COUNT_MAX; iCnt ++ )
	{
		if ( m_pPlayerChar[iCnt] )
		{
			delete m_pPlayerChar[iCnt];
		}
	}
	for ( int iCnt = 0; iCnt < BATTLE_ENEMY_COUNT_MAX; iCnt ++ )
	{
		if ( m_pEnemy[iCnt] )
		{	delete m_pEnemy[iCnt];	}
	}
	delete m_pField;
	delete m_pManipulator;
	delete m_pCommandMan;
	delete m_pDisplayMan;
}
//-----------------------------------------------------------------------------
// GetInstance
//-----------------------------------------------------------------------------
CBattleSystem *CBattleSystem::GetInstance( void )
{
	static CBattleSystem m_Instance;
	return &m_Instance;
}
//-----------------------------------------------------------------------------
// Initialize
// 
//-----------------------------------------------------------------------------
BOOL CBattleSystem::Initialize( void )
{
	// [hݒ
	m_nMode = MODE_INIT;

	// t[JE^Zbg
	m_dwSystemFrameCount = 0;

	return TRUE;
}
//-----------------------------------------------------------------------------
// Debug_SetupPartyInfo
// fobOpp[eBݒ
//-----------------------------------------------------------------------------
void CBattleSystem::Debug_SetupPartyInfo( stPlayerPartyInfo *pPartyInfo, stPlayerCharacterInfo *pCharaInfo )
{
	CTableAccess::LoadDefaultPlayerPartyTable( pPartyInfo, pCharaInfo );
}
//-----------------------------------------------------------------------------
// SetupBattleParam
//-----------------------------------------------------------------------------
BOOL CBattleSystem::SetupBattleParam( const stBattleParam *pParam )
{
	BOOL bRet = FALSE;
	
	HaltBattle();

	// n`Awiݒ
	bRet &= m_pField->Initialize( pParam->Field.nID );

	// p[eBݒ
	SetupPlayerParty( &pParam->Party, pParam->Chara );

	// Gl~[ݒ
	SetupEnemy( &pParam->Enemy );

	return bRet;
}
//-----------------------------------------------------------------------------
// SetupPlayerParty
//-----------------------------------------------------------------------------
void CBattleSystem::SetupPlayerParty( const stPlayerPartyInfo *pPartyInfo, const stPlayerCharacterInfo *pCharaInfo )
{
	// p[eBS̏
	m_PartyParam = *pPartyInfo;

	// LN^ʏ
	for ( int iCnt = 0; iCnt < BATTLE_CHAR_COUNT_MAX; iCnt ++ )
	{
		if ( m_pPlayerChar[iCnt] )
		{	delete m_pPlayerChar[iCnt];	}

		m_pPlayerChar[iCnt] = new CCharacter();

		// p[^ݒ
		m_pPlayerChar[iCnt]->m_Param = pCharaInfo[iCnt];

		// 摜Ǎ
		m_pPlayerChar[iCnt]->LoadGraph();

		// \ʒuݒ
		int nPosX;
		if ( m_pPlayerChar[iCnt]->m_Param.nBtlPos == BATTLE_POS_FRONT )
			nPosX = FRONT_CHAR_POS_X + (iCnt * 5);
		else
			nPosX = BACK_CHAR_POS_X + (iCnt * 5);
		m_pPlayerChar[iCnt]->SetDispPos( nPosX ,  45 + (iCnt * 30) );	// ToDo: l

		
		// Aj[VJn
		m_pPlayerChar[iCnt]->StartAnimation( ANI_PTN_WALK_AT_START );
	}
}
//-----------------------------------------------------------------------------
// SetupEnemy
//-----------------------------------------------------------------------------
void CBattleSystem::SetupEnemy( const stEnemyPartyInfo *pEnemy )
{
	// 
	for ( int iCnt = 0; iCnt < BATTLE_ENEMY_COUNT_MAX; iCnt ++ )
	{
		if ( m_pEnemy[iCnt] )
		{	delete m_pEnemy[iCnt];	}

		m_pEnemy[iCnt] = NULL;
	}

	// EnemyPartyTable.csṽ[h
	CTableAccess::LoadEnemyPartyTable( pEnemy->nID, m_pEnemy, &m_nEnemyCount );
}

//-----------------------------------------------------------------------------
// HaltBattle
//-----------------------------------------------------------------------------
void CBattleSystem::HaltBattle( void )
{
	for ( int iCnt = 0; iCnt < BATTLE_CHAR_COUNT_MAX; iCnt ++ )
	{
		if ( m_pPlayerChar[iCnt] )
		{
			m_pPlayerChar[iCnt]->Finalize();
		}
	}
	for ( int iCnt = 0; iCnt < BATTLE_ENEMY_COUNT_MAX; iCnt ++ )
	{
		if ( m_pEnemy[iCnt] )
		{
			m_pEnemy[iCnt]->Finalize();
		}
	}
	if ( m_pField )
	{
		m_pField->Finalize();
	}
}

//-----------------------------------------------------------------------------
// FrameMove
// XV
//-----------------------------------------------------------------------------
void CBattleSystem::FrameMove( void )
{
	// ANeBuł΃L[`FbN
	if ( GetActiveFlag() )
	{
		m_pManipulator->CheckKeyState();
	}

	// \}l[WXV
	m_pDisplayMan->FrameMove( m_pManipulator );

	m_dwSystemFrameCount ++;

	// [h
	if ( m_nMode == MODE_INIT )
	{	// 
		if ( m_pManipulator->IsAnyKeyPushed() )
		{
			// 퓬Jn
			StartBattle( NULL );
		}
		return;
	}

	if ( m_nMode == MODE_BATTLE )
	{	// 퓬

		// R}hs
		CommandProc();

		// SŃ`FbN
		BOOL bAliving = FALSE;
		for ( int iCnt = 0; iCnt < m_nEnemyCount; iCnt ++ )
		{
			if ( m_pEnemy[iCnt]->IsAliving() )
			{
				bAliving = TRUE;
				break;
			}
		}

		if ( !bAliving )
		{	// GS
			m_nMode = MODE_WIN;
		}
	}
	if ( m_nMode == MODE_WIN )
	{	// 
		m_pDisplayMan->ShowMessage( "" );
	}
}
//-----------------------------------------------------------------------------
// CommandProc
//-----------------------------------------------------------------------------
void CBattleSystem::CommandProc( void )
{
	// R}h
	stCommandInfo CommandInfo;
	if ( m_pCommandMan->GetCommand( &CommandInfo ) == COMMAND_NONE )
	{
		return;
	}

	switch ( CommandInfo.nID )
	{
	case COMMAND_PLAYER_ATTACK:		// 
		{
		// Param1Param2֕U

		// _[WvZ
		int nDamage = CalcDamage( COMMAND_PLAYER_ATTACK, CommandInfo.nParam1, CommandInfo.nParam2 );

		// UAj[Vݒ
		m_pPlayerChar[CommandInfo.nParam1]->StartAnimation( ANI_PTN_ATTACK );
		int iWaitFrame = m_pPlayerChar[CommandInfo.nParam1]->GetAnimationFrameToHpChange(ANI_PTN_ATTACK);
		// HP炵
		m_pEnemy[CommandInfo.nParam2]->ChangeHitPoint( nDamage, iWaitFrame );

		break;
		}
	case COMMAND_PLAYER_ITEM:		// ACe
		break;
	case COMMAND_PLAYER_DEFENCE:	// ڂ
		break;
	case COMMAND_PLAYER_PASS:		// pX
		break;
	case COMMAND_PLAYER_CHANGE:		// `FW
		break;
	case COMMAND_PLAYER_RUN:
		break;
	}
}
//-----------------------------------------------------------------------------
// CalcDamage
//-----------------------------------------------------------------------------
int CBattleSystem::CalcDamage( int nCommandID, int nParam1, int nParam2 )
{
	// ToDo
	return 30;
}

//-----------------------------------------------------------------------------
// StartBattle
//-----------------------------------------------------------------------------
BOOL CBattleSystem::StartBattle( const stBattleParam *pParam )
{
	BOOL bRet = FALSE;

	// 񏉊
	m_lstAction.clear();
	m_pCommandMan->RemoveAllCommand();

	if ( pParam == NULL )
	{	// ===== Kȏݒ	(Debug Only)

		stBattleParam BattleParam;
		ZeroMemory( &BattleParam, sizeof(BattleParam) );
		// n`Awiݒ
		BattleParam.Field.nID = 1;

		// p[eBݒ
		Debug_SetupPartyInfo( &BattleParam.Party, BattleParam.Chara );

		// GJEgGl~[ݒ
		BattleParam.Enemy.nID = 1;

		SetupBattleParam( &BattleParam );
	}
	else
	{
		SetupBattleParam( pParam );
	}

	// AACeʂł̃p[eB[C

	// ATBlݒ


	// \}l[W
	bRet &= m_pDisplayMan->Initialize( m_pPlayerChar, m_pEnemy, m_nEnemyCount, m_pField );
	
	// [hڍs
	m_nMode = MODE_BATTLE;

	// eXgp
	//m_pDisplayMan->StartNumberAnimation( 200,100, 300, GetColor(255,255,255) );

	return bRet;
}

//-----------------------------------------------------------------------------
// Render
// `
//-----------------------------------------------------------------------------
void CBattleSystem::Render( void )
{
	m_pDisplayMan->Render();
}

//-----------------------------------------------------------------------------
// AddCommand
// R}hǉ
//-----------------------------------------------------------------------------
void CBattleSystem::AddCommand( int nCommandID, int nParam1, int nParam2 )
{
	m_pCommandMan->AddCommand( nCommandID, nParam1, nParam2 );
}

//-----------------------------------------------------------------------------
// AddCommand
// R}hǉ
//-----------------------------------------------------------------------------
void CBattleSystem::AddActionablePlayer( int nIndex )
{
	// łɓo^ĂȂΓo^
	std::list<int>::iterator it = m_lstAction.begin();
	while( it != m_lstAction.end() )
	{
		if ( nIndex == (*it) )
		{	// łɓo^Ă Ȃ
			return;
		}
		++it;
	}
	m_lstAction.push_back( nIndex );
}