//=============================================================================
// DisplayManager.cpp
//=============================================================================
#include "Stdinc.h"
#include "BattleSystem.h"
#include "DisplayManager.h"
#include "EnemyPartyWindow.h"
#include "PlayerPartyWindow.h"
#include "CommandWindow.h"
#include "InfoWindow.h"
#include "Field.h"
#include "Character.h"
#include "Enemy.h"
#include "Manipulator.h"

//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CDisplayManager::CDisplayManager(void)
	:m_pPartyWnd( NULL ), m_pEnemyPtWnd( NULL ), m_pCommandWnd( NULL ), m_pSubCmdWnd( NULL ),m_pInfoWnd( NULL ),
	m_nDispMode( DISP_MODE_NONE ),
	m_nCurPosX( 0 ), m_nCurPosY( 0 ), m_nMarkPosX( 0 ), m_nMarkPosY( 0 ), m_nMarkIndex( -1 )

{
	// G̃nh
	m_nCursorHandle = 0;
	m_nMarkHandle = 0;
}
//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CDisplayManager::~CDisplayManager(void)
{
	DeleteGraph( m_nCursorHandle );
	DeleteGraph( m_nMarkHandle );

	if ( m_pInfoWnd )
	{
		delete m_pInfoWnd;
	}
	if ( m_pSubCmdWnd )
	{
		delete m_pSubCmdWnd;
	}
	if ( m_pCommandWnd )
	{
		delete m_pCommandWnd;
	}
	if ( m_pEnemyPtWnd )
	{
		delete m_pEnemyPtWnd;
	}
	if ( m_pPartyWnd )
	{
		delete m_pPartyWnd;
	}
}
//-----------------------------------------------------------------------------
// Initialize
// 
//-----------------------------------------------------------------------------
BOOL CDisplayManager::Initialize( CCharacter **pPlayerChar, CEnemy **pEnemy, int nEnemyCount, CField *pField )
{
	// Qƃ|C^ۑ
	m_pField = pField;

	memcpy( m_pPlayerChar, pPlayerChar, sizeof(m_pPlayerChar) );
	memcpy( m_pEnemy, pEnemy, sizeof(m_pEnemy) );
	m_nEnemyCount = nEnemyCount;

	// J[\ǂݍ
	if ( m_nCursorHandle )
	{
		DeleteGraph( m_nCursorHandle );
	}

	m_nCursorHandle = LoadGraph( CURSOR_FILE_NAME ) ;
	if ( m_nCursorHandle == DX_FAILED )
	{
		CUtility::tERROR("摜ǂݍݎs [%s]", CURSOR_FILE_NAME);
		return FALSE;
	}

	// }[NGǍ
	if ( m_nMarkHandle )
	{
		DeleteGraph( m_nMarkHandle );
	}
	m_nMarkHandle = LoadGraph( MARK_FILE_NAME ) ;
	if ( m_nMarkHandle == DX_FAILED )
	{
		CUtility::tERROR("摜ǂݍݎs [%s]", MARK_FILE_NAME);
		return FALSE;
	}

	m_lstWnd.clear();

	// Gp[eB\EBhE
	if ( m_pEnemyPtWnd )
	{
		delete m_pEnemyPtWnd;
	}
	m_pEnemyPtWnd = new CEnemyPartyWindow();
	m_pEnemyPtWnd->Create( 0/*X*/, 170/*Y*/, 130/**/, 70/**/, pEnemy, nEnemyCount );
	m_lstWnd.push_back( m_pEnemyPtWnd );

	// p[eB\EBhE
	if ( m_pPartyWnd )
	{
		delete m_pPartyWnd;
	}
	m_pPartyWnd = new CPlayerPartyWindow();
	m_pPartyWnd->Create( 130/*X*/, 170/*Y*/, 190/**/, 70/**/, pPlayerChar );
	m_lstWnd.push_back( m_pPartyWnd );

	// R}hEBhE
	if ( m_pCommandWnd )
	{
		delete m_pCommandWnd;
	}
	m_pCommandWnd = new CCommandWindow();
	m_pCommandWnd->Create(  25/*X*/, 170/*Y*/, 90/**/, 70/**/ );
	m_pCommandWnd->ShowWindow( FALSE );
	m_lstWnd.push_back( m_pCommandWnd );

	// TuR}hEBhE
	if ( m_pSubCmdWnd )
	{
		delete m_pSubCmdWnd;
	}
	m_pSubCmdWnd = new CCommandWindow();
	m_pSubCmdWnd->Create(  115/*X*/, 170/*Y*/, 90/**/, 70/**/ );
	m_pSubCmdWnd->ShowWindow( FALSE );
	m_lstWnd.push_back( m_pSubCmdWnd );

	// EBhE
	if ( m_pInfoWnd )
	{
		delete m_pInfoWnd;
	}
	m_pInfoWnd = new CInfoWindow();
	m_pInfoWnd->Create(  20/*X*/, 5/*Y*/, 280/**/, 25/**/ );
	m_pInfoWnd->ShowWindow( FALSE );
	m_lstWnd.push_back( m_pInfoWnd );

	// ANeBuEBhE
	m_pActiveWnd = m_pPartyWnd;

	// J[\[h
	m_nDispMode = DISP_MODE_NONE;

	// Iԏ
	m_nMarkIndex = -1;

	m_nCommandID = COMMAND_NONE;
	m_nCursorIndex = 0;

	return TRUE;
}
//-----------------------------------------------------------------------------
// FrameMove
// XV
//-----------------------------------------------------------------------------
void CDisplayManager::FrameMove( CManipulator *pMani )
{
	CBattleSystem *pSystem = CBattleSystem::GetInstance();

	if (  pSystem->GetMode() == MODE_INIT )
	{	// 
		return;
	}

	if ( pSystem->GetMode() == MODE_WIN )
	{	// 
		m_pCommandWnd->ShowWindow( FALSE );
		m_pSubCmdWnd->ShowWindow( FALSE );
		return;
	}

	
	// GXV
	for ( int iCnt = 0; iCnt < m_nEnemyCount; iCnt ++ )
	{
		m_pEnemy[iCnt]->FrameMove();
	}

	// XV
	for ( int iCnt = 0; iCnt < BATTLE_CHAR_COUNT_MAX; iCnt ++ )
	{
		m_pPlayerChar[iCnt]->FrameMove();
	}
	// p[eBEBhEXV
	m_pPartyWnd->FrameMove();

	// Gp[eBXgXV
	m_pEnemyPtWnd->FrameMove();

	// v[ ATBl~ϑ҂
	for ( int iCnt = 0; iCnt < BATTLE_CHAR_COUNT_MAX; iCnt ++ )
	{
		if ( iCnt == m_nMarkIndex )
			continue;

		if ( m_pPlayerChar[iCnt]->m_Param.nATBValue == ATB_VALUE_MAX )
		{
			pSystem->AddActionablePlayer( iCnt );
		}
	}

	// \[hƂ̏
	ModeProc( pMani );


}
//-----------------------------------------------------------------------------
// ModeProc
// J[\
//-----------------------------------------------------------------------------
void CDisplayManager::ModeProc( CManipulator *pMani )
{
	CBattleSystem *pSystem = CBattleSystem::GetInstance();

	switch ( m_nDispMode )
	{
	case DISP_MODE_NONE:				// J[\
	{
		// ANVXg`FbN
		int iIndex = pSystem->GetActionablePlayer();
		if ( iIndex != -1 )
		{
			m_nDispMode = DISP_MODE_PLAYER_COMMAND;

			m_nMarkPosX = m_pPlayerChar[iIndex]->m_nPosX +  2;
			m_nMarkPosY = m_pPlayerChar[iIndex]->m_nPosY - 12;
			m_nMarkIndex = iIndex;	// Ioۑ

			// ToDo : R}hb
			m_pCommandWnd->RemoveAllItems();
			m_pCommandWnd->AddItem( 0, 1, "" );
			m_pCommandWnd->AddItem( 3, 2, "ACe" );
			m_pCommandWnd->SetCurSel( 0 );

			m_pActiveWnd = m_pCommandWnd;
			m_pCommandWnd->ShowWindow( TRUE );
		}

		break;
	}
	case DISP_MODE_PLAYER_COMMAND:	// v[R}h҂

		// j[
		CheckMenuKeyInput( pMani );

		// EL[ŕʃj[J
		if ( pMani->GetKeyPushCnt( KEY_INPUT_RIGHT ) == 1 && ( m_pActiveWnd == m_pCommandWnd ) )
		{
			// TuR}hEBhEJ
			m_pSubCmdWnd->ShowWindow( FALSE );
			// b
			m_pSubCmdWnd->RemoveAllItems();
			m_pSubCmdWnd->AddItem( 0, 3, "ڂ" );
			m_pSubCmdWnd->AddItem( 1, 4, "pX" );
			m_pSubCmdWnd->AddItem( 2, 5, "`FW" );
			m_pSubCmdWnd->AddItem( 3, 6, "ɂ" );
			m_pSubCmdWnd->SetCurSel( 0 );

			m_pActiveWnd = m_pSubCmdWnd;
			m_pSubCmdWnd->ShowWindow( TRUE );
			break;
		}
		// L[ŕʃj[J
		if ( pMani->GetKeyPushCnt( KEY_INPUT_LEFT ) == 1 && ( m_pActiveWnd == m_pSubCmdWnd ) )
		{
			m_pActiveWnd = m_pCommandWnd;
			m_pSubCmdWnd->ShowWindow( FALSE );
			break;
		}
		// w{^
		if ( pMani->GetKeyPushCnt( KEY_INPUT_S ) == 1 )
		{
			m_nDispMode = DISP_MODE_NONE;		// v[I
			m_pSubCmdWnd->ShowWindow( FALSE );
			m_pCommandWnd->ShowWindow( FALSE );

			pSystem->AddActionablePlayer( m_nMarkIndex );
		}
		// L[
		if ( pMani->GetKeyPushCnt( KEY_INPUT_X ) == 1 )
		{
			m_nCommandID = m_pActiveWnd->NotifySubmit();
			// U
			if ( m_nCommandID == COMMAND_PLAYER_ATTACK )
			{
				m_nDispMode = DISP_MODE_ENEMY_SELECT;	// GI

				// E_獶ɃXLĈԋ߂G
				m_nCurPosX = CANVAS_WIDTH;
				m_nCurPosY = CANVAS_HEIGHT;
				m_nCursorIndex = GetEnemyCursor( KEY_INPUT_LEFT );
			}
			// pX
			else if ( m_nCommandID == COMMAND_PLAYER_PASS )
			{
				m_nDispMode = DISP_MODE_NONE;		// v[I
				m_pSubCmdWnd->ShowWindow( FALSE );
				m_pCommandWnd->ShowWindow( FALSE );

				pSystem->AddActionablePlayer( m_nMarkIndex );
			}
			// ACe
			else if ( m_nCommandID == COMMAND_PLAYER_ITEM )
			{
				m_nDispMode = DISP_MODE_SELECT_ITEM;	// ACeI
			}
			else
			{
				//COMMAND_DEFENCE		// ڂ
				//COMMAND_PASS			// pX
				//COMMAND_CHANGE		// `FW
				//COMMAND_RUN			// ɂ
				pSystem->AddCommand( m_nCommandID, m_nMarkIndex, 0  );	// R}ho^
			}
			break;
		}
		break;

	case DISP_MODE_ENEMY_SELECT:		// GI
		CheckEnemySelectKeyInput( pMani );
		// L[
		if ( pMani->GetKeyPushCnt( KEY_INPUT_X ) == 1 )
		{
			// U
			if ( m_nCommandID == COMMAND_PLAYER_ATTACK )
			{
				m_nDispMode = DISP_MODE_NONE;	// ATB҂
				m_pSubCmdWnd->ShowWindow( FALSE );
				m_pCommandWnd->ShowWindow( FALSE );
				pSystem->AddCommand( m_nCommandID, m_nMarkIndex, m_nCursorIndex  );	// R}ho^
				m_pPlayerChar[m_nMarkIndex]->ClearATBValue();
			}
		}
		break;
	case DISP_MODE_PLAYER_SELECT:		// v[I
		break;
	}
}
//-----------------------------------------------------------------------------
// CheckEnemySelectKeyInput
// GI̓`FbN
//-----------------------------------------------------------------------------
void CDisplayManager::CheckEnemySelectKeyInput( CManipulator *pMani )
{
	if ( pMani->GetKeyPushCnt( KEY_INPUT_UP ) == 1 )
	{
		m_nCursorIndex = GetEnemyCursor( KEY_INPUT_UP );
	}
	if ( pMani->GetKeyPushCnt( KEY_INPUT_DOWN ) == 1 )
	{
		m_nCursorIndex = GetEnemyCursor( KEY_INPUT_DOWN );
	}
	if ( pMani->GetKeyPushCnt( KEY_INPUT_LEFT ) == 1 )
	{
		m_nCursorIndex = GetEnemyCursor( KEY_INPUT_LEFT );
	}
	if ( pMani->GetKeyPushCnt( KEY_INPUT_RIGHT ) == 1 )
	{
		m_nCursorIndex = GetEnemyCursor( KEY_INPUT_RIGHT );
	}
}
//-----------------------------------------------------------------------------
// GetEnemyCursor
// m_nCurPosX, m_nCurPosY ̈ʒu nKeyCode̕
// ԋ߂ĂGT
//-----------------------------------------------------------------------------
int CDisplayManager::GetEnemyCursor( int nKeyCode )
{
	int nIndex = -1;
	int nDiffY = CANVAS_HEIGHT;
	int nDiffX = CANVAS_WIDTH;
	for ( int iCnt = 0; iCnt < BATTLE_ENEMY_COUNT_MAX; iCnt ++ )
	{
		if ( !m_pEnemy[iCnt] || m_pEnemy[iCnt]->m_nHP <= 0 )
			continue;	// HP 0̓G͏O

		if ( iCnt == m_nCursorIndex )
			continue;	// IĂ̂O

		if ( nKeyCode == KEY_INPUT_LEFT || nKeyCode == KEY_INPUT_UP )
		{	// 
			if ( m_nCurPosX - m_pEnemy[iCnt]->m_nPosX < nDiffX && 0 <= m_nCurPosX - m_pEnemy[iCnt]->m_nPosX )
			{
				nDiffX = m_nCurPosX - m_pEnemy[iCnt]->m_nPosX;
				nIndex = iCnt;
			}
		}
		if ( nKeyCode == KEY_INPUT_RIGHT || nKeyCode == KEY_INPUT_DOWN )
		{	// E
			if ( m_pEnemy[iCnt]->m_nPosX - m_nCurPosX < nDiffX && 0 <= m_pEnemy[iCnt]->m_nPosX - m_nCurPosX )
			{
				nDiffX = m_pEnemy[iCnt]->m_nPosX - m_nCurPosX;
				nIndex = iCnt;
			}
		}
#if 0	// ㉺L[͍EL[Ɠ̓ɂĂ݂
		if ( nKeyCode == KEY_INPUT_UP )
		{	// 
			if ( m_nCurPosY - m_pEnemy[iCnt]->m_nPosY < nDiffY && 0 <= m_nCurPosY - m_pEnemy[iCnt]->m_nPosY )
			{
				nDiffY = m_nCurPosY - m_pEnemy[iCnt]->m_nPosY;
				nIndex = iCnt;
			}
		}

		if ( nKeyCode == KEY_INPUT_DOWN )
		{	// 
			if ( m_pEnemy[iCnt]->m_nPosY - m_nCurPosY < nDiffY && 0 <= m_pEnemy[iCnt]->m_nPosY - m_nCurPosY )
			{
				nDiffY = m_pEnemy[iCnt]->m_nPosY - m_nCurPosY;
				nIndex = iCnt;
			}
		}
#endif
	}

	if ( nIndex == -1 )
	{	// 
		return m_nCursorIndex;
	}
	return nIndex;
}

//-----------------------------------------------------------------------------
// CheckMenuKeyInput
// ̓`FbN
//-----------------------------------------------------------------------------
void CDisplayManager::CheckMenuKeyInput( CManipulator *pMani )
{
	int CHECK_KEYS[] = {
		KEY_INPUT_UP, KEY_INPUT_DOWN, /* KEY_INPUT_X, KEY_INPUT_Z, KEY_INPUT_S */
	};

	for ( int iCnt = 0; iCnt < sizeof(CHECK_KEYS) / sizeof(CHECK_KEYS[0] ); iCnt ++ )
	{
		if ( pMani->GetKeyPushCnt( CHECK_KEYS[iCnt] ) == 1 )
		{
			m_pActiveWnd->NotifyKeyInput( CHECK_KEYS[iCnt] );
		}
	}
}

//-----------------------------------------------------------------------------
// Render
// ZI[_[`
//-----------------------------------------------------------------------------
void CDisplayManager::Render( void )
{
	CBattleSystem *pSystem = CBattleSystem::GetInstance();

	if (  pSystem->GetMode() == MODE_INIT )
	{	// 
		DrawString( 105, 115, "waiting start message", STRING_COLOR );
		DrawString(  70, 135, "press any key to start with .\\default.csv", STRING_COLOR );
		return;
	}

	using namespace std;

	// wi`
	m_pField->Render();

	// G`
	for ( int iCnt = 0; iCnt < m_nEnemyCount; iCnt ++ )
	{
		if ( m_pEnemy[iCnt]->IsAliving() )
		{
			m_pEnemy[iCnt]->Render();
		}
	}

	// `
	for ( int iCnt = 0; iCnt < BATTLE_CHAR_COUNT_MAX; iCnt ++ )
	{
		m_pPlayerChar[iCnt]->Render();
	}

	// EBhE`
	list<CFrameWindow *>::iterator it = m_lstWnd.begin();
	while( it != m_lstWnd.end() )
	{
		(*it)->Render();
		++it;
	}

	if ( pSystem->GetMode() == MODE_BATTLE )
	{
		// J[\`
		if ( m_nDispMode == DISP_MODE_PLAYER_COMMAND )
		{
			// R}hj[I J[\\
			m_pActiveWnd->GetCursorDispPos( &m_nCurPosX, &m_nCurPosY );
			DrawGraph( m_nCurPosX, m_nCurPosY, m_nCursorHandle, TRUE );
		}
		else if ( m_nDispMode == DISP_MODE_ENEMY_SELECT )
		{
			m_pEnemy[m_nCursorIndex]->GetCursorDispPos( &m_nCurPosX, &m_nCurPosY );
			DrawGraph( m_nCurPosX, m_nCurPosY, m_nCursorHandle, TRUE );
		}

		// }[N`
		if ( m_nDispMode == DISP_MODE_PLAYER_COMMAND )
		{
			// R}hj[I }[N`
			DrawGraph( m_nMarkPosX, m_nMarkPosY, m_nMarkHandle, TRUE );
		}
	}
}
//-----------------------------------------------------------------------------
// ShowMessage
//-----------------------------------------------------------------------------
void CDisplayManager::ShowMessage( const char *lpzText )
{
	m_pInfoWnd->SetMessage( lpzText );
	m_pInfoWnd->ShowWindow( TRUE );
}
